package com.atguigu.common.utlis;

import com.atguigu.model.system.SysMenu;
import com.atguigu.vo.system.MetaVo;
import com.atguigu.vo.system.RouterVo;
import org.springframework.util.CollectionUtils;
import org.springframework.util.StringUtils;

import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.stream.Collectors;

public class MenuHelper {
    /**
     * 构建树型结构
     *
     * @param sysMenuList 系统菜单列表
     * @return 菜单结构列表
     */
    public static List<SysMenu> buildTree(List<SysMenu> sysMenuList) {
        ArrayList<SysMenu> tress = new ArrayList<>();
        // 把菜单数据进行遍历
        sysMenuList.forEach(sysMenu -> {
            if (sysMenu.getParentId() == 0) tress.add(getChildren(sysMenu, sysMenuList));
        });
        return tress;
    }

    /**
     * 构建递归循环
     *
     * @param sysMenu     当前菜单对象
     * @param sysMenuList 全部菜单列表
     * @return 菜单
     */
    private static SysMenu getChildren(SysMenu sysMenu, List<SysMenu> sysMenuList) {
        // 遍历所有菜单数据，判断id和parentID对应关系
        sysMenu.setChildren(new ArrayList<>());

        sysMenuList.forEach(menu -> {
            if (menu.getChildren() == null) menu.setChildren(new ArrayList<>());
            if (sysMenu.getId().equals(menu.getParentId())) {
                sysMenu.getChildren().add(getChildren(menu, sysMenuList));
            }
        });

        return sysMenu;
    }

    /**
     * 封装路由菜单
     *
     * @param menuTreeList 菜单树型结构
     * @return 路由菜单列表
     */
    public static List<RouterVo> buildRouter(List<SysMenu> menuTreeList) {
        List<RouterVo> routers = new LinkedList<>();
        menuTreeList.forEach(menu -> {
            RouterVo router = new RouterVo();
            router.setHidden(false);
            router.setAlwaysShow(false);
            router.setPath(getRouterPath(menu));
            router.setComponent(menu.getComponent());
            router.setMeta(new MetaVo(menu.getName(), menu.getIcon()));
            List<SysMenu> children = menu.getChildren();
            // 如果当前是菜单，需将按钮对应的路由加载出来，如：“角色授权”按钮对应的路由在“系统管理”下面
            if (menu.getType() == 1) {
                List<SysMenu> hiddenMenuList = children.stream().filter(item -> !StringUtils.isEmpty(item.getComponent())).collect(Collectors.toList());
                for (SysMenu hiddenMenu : hiddenMenuList) {
                    RouterVo hiddenRouter = new RouterVo();
                    hiddenRouter.setHidden(true);
                    hiddenRouter.setAlwaysShow(false);
                    hiddenRouter.setPath(getRouterPath(hiddenMenu));
                    hiddenRouter.setComponent(hiddenMenu.getComponent());
                    hiddenRouter.setMeta(new MetaVo(hiddenMenu.getName(), hiddenMenu.getIcon()));
                    routers.add(hiddenRouter);
                }
            } else {
                if (!CollectionUtils.isEmpty(children)) {
                    router.setAlwaysShow(true);
                    router.setChildren(buildRouter(children));
                }
            }
            routers.add(router);
        });
        return routers;
    }

    /**
     * 获取路由地址
     *
     * @param menu 菜单信息
     * @return 路由地址
     */
    private static String getRouterPath(SysMenu menu) {
        String routerPath = "/" + menu.getPath();
        if (menu.getParentId().intValue() != 0) {
            routerPath = menu.getPath();
        }
        return routerPath;
    }
}
