package com.atguigu.auth.service;

import com.atguigu.model.system.SysMenu;
import com.atguigu.vo.system.AssginMenuVo;
import com.atguigu.vo.system.RouterVo;
import com.baomidou.mybatisplus.extension.service.IService;

import java.util.List;

/**
 * <p>
 * 菜单表 服务类
 * </p>
 *
 * @author bunny
 * @since 2024-04-23
 */
public interface SysMenuService extends IService<SysMenu> {

    /**
     * 获取菜单
     *
     * @return 菜单列表
     */
    List<SysMenu> findNodes();

    /**
     * 删除菜单
     *
     * @param id 菜单id
     */
    void removeMenuById(Long id);

    /**
     * 根据角色获取菜单
     *
     * @param roleId 角色id
     * @return 菜单列表
     */
    List<SysMenu> findSysMenuByRoleId(Long roleId);

    /**
     * 给角色分配权限
     *
     * @param assginMenuVo 分配条件
     */
    void doAssign(AssginMenuVo assginMenuVo);

    /**
     * 根据用户ID查询用户可以操作的菜单列表
     *
     * @param userId 用户ID
     * @return 路由信息
     */
    List<RouterVo> findUserMenuListByUserId(Long userId);

    /**
     * 根据用户ID查询用户按钮权限
     *
     * @param userId 用户ID
     * @return 按钮权限
     */
    List<String> findUserPermsList(Long userId);
}
